module Rouge
  module Lexers
    class Cigma < RegexLexer
      title 'Cigma'
      desc "The Cigma programming language"
      tag 'cigma'
      filenames '*.cm'

      state :whitespace do
        rule %r/\s+/, Text::Whitespace
      end

      state :reserved do
        rule %r/\+?\$/, Keyword::Declaration
        rule %r/\-\>/, Keyword
        rule %r/\b(new|this|delete|sizeof|alignof|typeid|long|unsigned|volatile|if|else|for|while|continue|break|switch|goto|throw|try|catch)\b/, Keyword::Reserved
      end

      state :number do
        rule %r/[+-]?[0-9]+e[+-]?[0-9]+/, Literal::Number
        rule %r/[0-9]*[.][0-9]+/, Literal::Number
        rule %r/[0-9]+/, Literal::Number
        rule %r/[0-9]+x[_0-9A-Fa-f]+/, Literal::Number
      end

      state :string do
        rule %r/[^"\\]+/, Literal::String::Double
        rule %r/\\./, Literal::String::Escape
        rule %r/"/, Literal::String::Double, :pop!
      end

      state :id do
        rule %r/(\.)([a-zA-Z_][a-zA-Z0-9_]*)\b/ do
          groups Punctuation, Name::Variable::Instance
        end
        rule %r/(\:)([a-zA-Z_][a-zA-Z0-9_]*)\b/ do
          groups Punctuation, Keyword::Type
        end
        rule %r/(\/)([a-zA-Z_][a-zA-Z0-9_]*)\b/ do
          groups Punctuation, Keyword::Namespace
        end
        rule %r/(\:\:)([a-zA-Z_][a-zA-Z0-9_]*)\b/ do
          groups Punctuation, Keyword::Type
        end
        rule %r/(\@)([a-zA-Z_][a-zA-Z0-9_]*)\b/ do
          groups Punctuation, Name::Attribute
        end
      end

      state :op do
        rule %r/\&|\!|\^|\~|\=|\'|\*|\*\=|\/|\/\=|\%|\%\=|\+|\+\=|\-|\-\=|\||\&\=|\+\+|\-\-|\&\&|\|\||\<\<|\<\<=|\>\>|\>\>=|\=\=|\<\>|\<|\<\=|\>|\>\=/, Operator
      end

      state :userop do
        rule %r/[^`]+/, Literal::String::Backtick
        rule %r/`/, Literal::String::Backtick, :pop!
      end

      state :type do
        rule %r/\'\.\(/ do
          token Punctuation
          push :type
        end
        rule %r/\(/ do
          token Punctuation
          push :paren
        end
        rule %r/[\.\!\?\&\^\']+/, Keyword::Type
        mixin :number
        mixin :id
        rule %r/\b[a-zA-Z_][a-zA-Z0-9_]*\b/, Keyword::Type
        rule %r/\)/, Punctuation, :pop!
        rule %r/./, Text, :pop!
      end

      state :attribute do
        rule %r/\b([a-zA-Z_][a-zA-Z0-9_]*)\b/, Name::Attribute
        mixin :common
        rule %r/{/ do
          token Punctuation
          push :attribute
        end
        rule %r/}/, Punctuation, :pop!
        rule %r/./, Text
    end

      state :paren do
        mixin :common
        rule %r/\)/, Punctuation, :pop!
        rule %r/./, Text
      end

      state :common do
        mixin :whitespace
        rule %r/#.*/, Comment
        mixin :reserved
        mixin :number
        mixin :id
        mixin :op
        rule %r/@{/ do
          token Punctuation
          push :attribute
        end
        rule %r/"/ do
          token Literal::String::Double
          push :string
        end
        rule %r/`/ do
          token Literal::String::Backtick
          push :userop
        end
        rule %r/\(/ do
          token Punctuation
          push :paren
        end
        rule %r/::/ do
          token Punctuation
          push :type
        end
        rule %r/:/ do
          token Punctuation
          push :type
        end
        rule %r/\b([a-zA-Z_][a-zA-Z0-9_]*)\b/, Name
      end

      state :root do
        mixin :common
        rule %r/./, Text
      end
    end
  end
end
